package cl.frn.pooseven;

import io.github.palexdev.materialfx.controls.MFXTableColumn;
import io.github.palexdev.materialfx.controls.cell.MFXTableRowCell;
import io.github.palexdev.materialfx.filter.EnumFilter;
import io.github.palexdev.materialfx.filter.IntegerFilter;
import io.github.palexdev.materialfx.filter.LongFilter;
import io.github.palexdev.materialfx.filter.StringFilter;
import javafx.application.Platform;
import javafx.collections.FXCollections;
import javafx.collections.ObservableList;
import javafx.geometry.Pos;
import org.eclipse.paho.client.mqttv3.IMqttMessageListener;
import org.eclipse.paho.client.mqttv3.MqttException;
import org.eclipse.paho.client.mqttv3.MqttMessage;

import java.util.Comparator;
import java.util.Date;
import java.util.UUID;

public class AccessController {

    AppController app;

    AccessController(AppController app) {

        this.app = app;

        app.acceptButton.setOnAction(_ -> onAcceptButtonClick());
        app.acceptButton.setOnAction(_ -> onDenyButtonClick());

        setupAccessTable();
        setupUsersTable();

        app.access_table.autosizeColumnsOnInitialization();
        app.users_table.autosizeColumnsOnInitialization();

    }

    protected void setupAccessTable() {

        app.access_table.setVisible(false);

        MFXTableColumn<AccessEvent> timestampColumn = new MFXTableColumn<>("Marca de tiempo", true, Comparator.comparing(AccessEvent::getTimestamp));
        MFXTableColumn<AccessEvent> usernameColumn = new MFXTableColumn<>("Usuario", true, Comparator.comparing(AccessEvent::getName));
        MFXTableColumn<AccessEvent> accessColumn = new MFXTableColumn<>("Acceso", true, Comparator.comparing(AccessEvent::getTypeOrdinal));
        timestampColumn.setMinWidth(150);
        timestampColumn.setRowCellFactory(_ -> new MFXTableRowCell<>(AccessEvent::formattedDate));
        usernameColumn.setRowCellFactory(_ -> new MFXTableRowCell<>(AccessEvent::getName));
        accessColumn.setRowCellFactory(_ -> new MFXTableRowCell<>(AccessEvent::getType));

        app.access_table.getTableColumns().addAll(timestampColumn, usernameColumn, accessColumn);
        app.access_table.getFilters().addAll(
                new StringFilter<>("Nombre", AccessEvent::getName),
                new EnumFilter<>("Tipo", AccessEvent::getType, AccessEvent.AccessType.class)
        );

        app.access_table.setItems(Data.accessEvents);
        app.access_table.setVisible(true);

    }


    protected void setupUsersTable() {


        MFXTableColumn<User> idColumn = new MFXTableColumn<>("Id", true, Comparator.comparing(User::getId));
        MFXTableColumn<User> usernameColumn = new MFXTableColumn<>("Nombre ", true, Comparator.comparing(User::getName));
        MFXTableColumn<User> cardColumn = new MFXTableColumn<>("Card ID", true, Comparator.comparing(User::getCard_id));

        idColumn.setRowCellFactory(_ -> new MFXTableRowCell<>(User::getId));
        usernameColumn.setRowCellFactory(_ -> new MFXTableRowCell<>(User::getName));
        cardColumn.setRowCellFactory(_ -> new MFXTableRowCell<>(User::stringCard_id) {{
            setAlignment(Pos.CENTER_RIGHT);
        }});
        cardColumn.setAlignment(Pos.CENTER_RIGHT);

        app.users_table.getTableColumns().addAll(idColumn, usernameColumn, cardColumn);
        app.users_table.getFilters().addAll(
                new StringFilter<>("Nombre", User::getName),
                new IntegerFilter<>("ID", User::getId),
                new LongFilter<>("CardID", User::getCard_id)
        );

        app.users_table.setItems(Data.users);


        app.mqtt.registerSubscription(

                new MQTTController.Subscription(

                        STR."device/\{FakeDoorController.device_id}/access/request", this::handleMqtt

                )
        );

    }

    private void handleMqtt(String s, MqttMessage mqttMessage) {

        Platform.runLater(new Runnable(){
            @Override
            public void run() {

                for (User part : Data.users) {
                    if (part.getCard_id() == Long.parseLong(mqttMessage.toString())) {
                        AccessEvent e = new AccessEvent();
                        Date d = new Date();
                        e.timestamp = d;
                        e.id = UUID.randomUUID();
                        e.user = part;
                        e.type = AccessEvent.AccessType.ENTRY;

                        System.out.println(part);

                        Data.accessEvents.add(e);

                        try {
                            app.mqtt.client.publish(STR."device/\{FakeDoorController.device_id}/access/manage", new MqttMessage("OPEN".getBytes()));
                        } catch (MqttException ex) {
                            ex.printStackTrace();
                        }
                        break;
                    }
                }
            }
        });


    }

    protected void onAcceptButtonClick() {

        // TODO accept access request
    }

    protected void onDenyButtonClick() {
        // TODO dey access

    }


}
