

from config import *
from lib.bmp280 import *
from lib.mqtt import MQTTClient
from machine import Pin, I2C
from functions import *

import network
import utime

# create a message queue
# import queue


class Door:
    # TODO: add opening msg to send queue

    OPEN = 1
    CLOSED = 0

    CLOSE_ACTION_MESSAGE = 0
    OPEN_ACTION_MESSAGE = 1

    def __init__(self):
        self.RED_LED = Pin(LED_1_PIN, Pin.OUT)
        self.GREEN_LED = Pin(LED_2_PIN, Pin.OUT)
        pass

    state = CLOSED

    def isOpen(self) -> bool:
        return bool(self.state)

    def open(self) -> None:
        self.RED_LED.off()
        self.GREEN_LED.on()
        beep()
        self.state = self.OPEN

    def close(self) -> None:
        self.RED_LED.on()
        self.GREEN_LED.off()
        beep()
        self.state = self.CLOSED
    def handle(self, msg):
        if msg == str(self.OPEN_ACTION_MESSAGE):
            self.open()
        elif msg == str(self.CLOSE_ACTION_MESSAGE):
            self.close()


class App:

    def __init__(self):
        print("init app")

        bus = I2C(0, sda=SDA_PIN, scl=SCK_PIN, freq=400000)

        self.temp_sensor = BMP280(bus)
        self.temp_sensor.use_case(BMP280_CASE_INDOOR)

        self.door = Door()

        # init wifi
        self.initWifi()

        if self.wlan.isconnected():
            self.initMqtt()

            # init mqtt

        # init sensors
        # init leds
        # init heartbeat
        # init callback
        #


    def initMqtt(self):
        self.client = MQTTClient(
            client_id=DEVICE_ID,
            server=MQTT_SERVER_IP,
            port=MQTT_SERVER_PORT,
            keepalive=300
        )
        self.client.connect()
        self.client.set_callback(self.callback)

    def initWifi(self):
        self.wlan = network.WLAN(network.STA_IF)
        self.wlan.active(True)
        self.wlan.connect(WIFI_SSID, WIFI_PASS)
        while not self.wlan.isconnected():
            utime.sleep(1)
        print(self.wlan.ifconfig())

    lastTime = 0
    def sendTemperature(self):
        if utime.ticks_ms() >= self.lastTime + 100:
            self.client.publish(
                f"device/{DEVICE_ID}/sensors/temp", str(self.temp_sensor.temperature))

    def loop(self):
        try:
            while True:
                self.sendTemperature()
                self.client.check_msg()
                sleep_ms(100) # TODO remove


        except KeyboardInterrupt:
            print("key interrupt")


    def handleDoorAction(self, msg):
        self.door.handle(msg)

    def ping(self, msg):
        self.client.publish(f"device/{DEVICE_ID}/pong", "hi: "+msg)

    actions = {
        f"device/{DEVICE_ID}/ping": ping,
        f"device/{DEVICE_ID}/action/door": handleDoorAction,
    }

    def callback(self, topic_bytes: bytes, msg_bytes: bytes):
        # switch by topic
        topic = topic_bytes.decode()
        msg = msg_bytes.decode()

        action = self.actions.get(topic)


        if action:
            print(f"{topic=} {msg=}")
            action(self, msg)
        else:
            print(f"{utime}unhandled mqtt topic: {topic=}")
